import * as THREE from '../build/three.module.js';
import Stats from './jsm/libs/stats.module.js';

class App {
    constructor() {
        const domWebGL = document.createElement('div');
        document.body.appendChild(domWebGL);
        this._domWebGL = domWebGL;

        const renderer = new THREE.WebGLRenderer({ antialias: true });
        //renderer.outputEncoding = THREE.sRGBEncoding;
        renderer.setPixelRatio(window.devicePixelRatio);
        //renderer.shadowMap.enabled =  true;
        domWebGL.appendChild(renderer.domElement);

        this._renderer = renderer;

        const scene = new THREE.Scene();
        scene.background = new THREE.Color(0xffffff);
        this._scene = scene;

        this._clock = new THREE.Clock();

        this._setupCamera();
        this._setupLight();
        this._setupModel();
        //this._setupInteractions();
        this._setupGUI();
        //this._setupControls();
        //this._setupHelper();

        window.onresize = this.resize.bind(this);

        this.resize();
        this.render()
    }

    _setupInteractions() {
        //.
    }

    _setupPostprocessing() {
        //.
    }

    _setupModel() {
        const canvas = document.createElement("canvas");
        canvas.width = 128;
        canvas.height = 128;

        const context = canvas.getContext("2d");
        const gradient = context.createRadialGradient(canvas.width/2, canvas.height/2, 0, 
            canvas.width/2, canvas.height/2, canvas.width/2);
        gradient.addColorStop(0.1, "rgba(210,210,210,1");
        gradient.addColorStop(1, "rgba(255,255,255,1");
        context.fillStyle = gradient;
        context.fillRect(0, 0, canvas.width, canvas.height);

        const shadowTexture = new THREE.CanvasTexture(canvas);
        const shadowMaterial = new THREE.MeshBasicMaterial({ map: shadowTexture });
        const shadowGeo = new THREE.PlaneGeometry(300, 300, 1, 1);

        let shadowMesh;

        shadowMesh = new THREE.Mesh(shadowGeo, shadowMaterial);
        shadowMesh.position.y = -250;
        shadowMesh.rotation.x = -Math.PI / 2;
        this._scene.add(shadowMesh);

        shadowMesh = new THREE.Mesh(shadowGeo, shadowMaterial);
        shadowMesh.position.y = -250;
        shadowMesh.position.x = -400;
        shadowMesh.rotation.x = -Math.PI / 2;
        this._scene.add(shadowMesh);

        shadowMesh = new THREE.Mesh(shadowGeo, shadowMaterial);
        shadowMesh.position.y = -250;
        shadowMesh.position.x = 400;
        shadowMesh.rotation.x = -Math.PI / 2;
        this._scene.add(shadowMesh);

        const radius = 200;
        const geometry1 = new THREE.IcosahedronGeometry(radius, 1);
        const count = geometry1.attributes.position.count;
        geometry1.setAttribute("color", new THREE.BufferAttribute(new Float32Array(count*3), 3));
        const geometry2 = geometry1.clone();
        const geometry3 = geometry1.clone();
        const color = new THREE.Color();
        const positions1 = geometry1.attributes.position;
        const positions2 = geometry2.attributes.position;
        const positions3 = geometry3.attributes.position;
        const colors1 = geometry1.attributes.color;
        const colors2 = geometry2.attributes.color;
        const colors3 = geometry3.attributes.color;

        for(let i=0; i<count; i++) {
            color.setHSL((positions1.getY(i) / radius+1) / 2, 1.0, 0.5);
            colors1.setXYZ(i, color.r, color.b, color.b);

            color.setHSL(0, (positions2.getY(i)/radius+1)/2, 0.5);
            colors2.setXYZ(i, color.r, color.g, color.b);

            color.setRGB(0.5, 0.8-(positions3.getY(i)/radius+1)/2, 0);
            colors3.setXYZ(i, color.r, color.g, color.b);
        }

        const material = new THREE.MeshPhongMaterial({
            color: 0xffffff,
            flatShading: true,
            vertexColors: true,
            shininess: 0
        });

        const wireframeMaterial = new THREE.MeshBasicMaterial({color: 0x000000, wireframe: true, transparent: true });
        let mesh = new THREE.Mesh(geometry1, material);
        let wireframe = new THREE.Mesh(geometry1, wireframeMaterial);
        mesh.add(wireframe);
        mesh.position.x = -400;
        mesh.rotation.x = -1.87;
        this._scene.add(mesh);

        mesh = new THREE.Mesh(geometry2, material);
        wireframe = new THREE.Mesh(geometry2, wireframeMaterial);
        mesh.add(wireframe);
        mesh.position.x = 400;
        this._scene.add(mesh);

        mesh = new THREE.Mesh(geometry3, material);
        wireframe = new THREE.Mesh(geometry3, wireframeMaterial);
        mesh.add(wireframe);
        this._scene.add(mesh);


    }

    _setupHelper() {
        //.
    }

    _setupGUI() {
        const stats = new Stats();
        this._domWebGL.appendChild(stats.dom);
        this._stats = stats;

        document.addEventListener("mousemove", this._onDocumentMouseMove.bind(this));
    }

    _onDocumentMouseMove(event) {
        this._mouseX = (event.clientX - window.innerWidth / 2);
        this._mouseY = (event.clientY - window.innerHeight / 2);
    }

    _setupCamera() {
        const camera = new THREE.PerspectiveCamera(30, window.innerWidth / window.innerHeight, 1, 10000);
        camera.position.z = 1800;

        this._camera = camera;
    }

    _setupControls() {
        //.
    }

    _setupLight() {
        const light = new THREE.DirectionalLight(0xffffff);
        light.position.set(0, 0, 1);
        this._scene.add(light);
    }

    update() {
        if(this._mouseX && this._mouseY) {
            this._camera.position.x += (this._mouseX - this._camera.position.x) * 0.05;
            this._camera.position.y += (this._mouseY - this._camera.position.y) * 0.05;
            this._camera.lookAt(this._scene.position);
        }

        this._stats.update();
    }

    render() {
        requestAnimationFrame(this.render.bind(this));
        
        this.update();
        this._renderer.render(this._scene, this._camera);
    }

    resize() {
        this._camera.aspect = window.innerWidth / window.innerHeight;
        this._camera.updateProjectionMatrix();
        this._renderer.setSize(window.innerWidth, window.innerHeight);
    }
}

window.onload = function () {
    new App();
}